@@ -47,6 +47,9 @@ char __ulp_metadata_buffer[ULP_METADATA_BUF_LEN] = { 0 };
 struct ulp_metadata *__ulp_metadata_ref = NULL;
 struct ulp_detour_root *__ulp_root = NULL;
 
+/* current libpulp version.  */
+const char __ulp_version[] = VERSION;
+
 /* clang-format off */
 
 /** Intel endbr64 instruction optcode.  */
@@ -420,7 +423,7 @@ load_so(char *obj)
 {
   void *patch_obj;
 
-  patch_obj = dlopen(obj, RTLD_LAZY);
+  patch_obj = dlopen(obj, RTLD_NOW);
   if (!patch_obj) {
     WARN("Unable to load shared object %s: %s.", obj, dlerror());
     return NULL;
@@ -456,7 +459,7 @@ parse_metadata(struct ulp_metadata *ulp)
     goto metadata_clean;
 
   if (!load_so_handlers(ulp)) {
-    ret = errno;
+    ret = EDLOPEN;
     goto metadata_clean;
   }
 
@@ -727,18 +730,24 @@ ulp_apply_all_units(struct ulp_metadata *ulp)
          symbol's address by its name.  */
       patch_address =
           (uintptr_t)load_so_symbol(ref->reference_name, ulp->so_handler);
+
+      if (patch_address == 0) {
+        return ENONEWFUNC;
+      }
+
       ref->patch_offset = patch_address - patch_base;
     }
     else {
       patch_address = patch_base + ref->patch_offset;
     }
+
     if (ref->tls) {
       tls_index ti = { .ti_module = tls_idx, .ti_offset = ref->target_offset };
-      memcpy((void *)patch_address, &ti, sizeof(ti));
+      insnq_insert_write((void *)patch_address, sizeof(ti), &ti);
     }
     else {
       uintptr_t target_address = target_base + ref->target_offset;
-      memcpy((void *)patch_address, &target_address, sizeof(void *));
+      insnq_insert_write((void *)patch_address, sizeof(void *), &target_address);
     }
     ref = ref->next;
   }
@@ -828,6 +837,11 @@ ulp_state_update(struct ulp_metadata *ulp)
     unit = unit->next;
   }
 
+  /* Insert timestamp.  */
+  struct timespec t;
+  clock_gettime(CLOCK_REALTIME, &t);
+  a_patch->timestamp = t.tv_sec;
+
   /* leave last on top of list to optmize revert */
   prev_patch = __ulp_state.patches;
   __ulp_state.patches = a_patch;
