@@ -189,12 +189,8 @@ release_ulp_dynobj(struct ulp_dynobj *obj)
 
   for (; obj != NULL; obj = nexto) {
     nexto = obj->next;
-
-    if (obj->filename) {
-      free(obj->filename);
-    }
-    if (obj->thread_states)
-      free(obj->thread_states);
+    FREE_AND_NULLIFY(obj->filename);
+    FREE_AND_NULLIFY(obj->thread_states);
     free(obj);
   }
 }
@@ -848,16 +844,21 @@ get_libpulp_extern_symbols(struct ulp_dynobj *obj, int pid)
         obj->enable_disable_patching = ehdr_addr + sym.st_value;
         bitfield |= (1 << 8);
       }
-
       else if (!strcmp(remote_name, "_insn_queue")) {
         obj->insn_queue = ehdr_addr + sym.st_value;
         bitfield |= (1 << 9);
       }
+      else if (!strcmp(remote_name, "_version")) {
+        char str[32];
+        read_string_allocated(str, sizeof(str), pid, ehdr_addr + sym.st_value);
+        obj->libpulp_version = ulp_version_from_string(str);
+        bitfield |= (1 << 10);
+      }
     }
 
     dynsym_addr += sizeof(sym);
 
-    if (bitfield == 0x3FF)
+    if (bitfield == 0x7FF)
       break;
   }
 
@@ -1058,7 +1059,7 @@ parse_main_dynobj(struct ulp_process *process)
   }
 
   /* calloc initializes all to zero */
-  obj = calloc(sizeof(struct ulp_dynobj), 1);
+  obj = calloc(1, sizeof(struct ulp_dynobj));
   if (!obj) {
     DEBUG("unable to allocate memory.");
     return ENOMEM;
@@ -1158,7 +1159,7 @@ get_libname_dynobj(struct ulp_process *process, struct link_map *link_map_addr)
   int pid = process->pid;
 
   /* calloc initializes all to zero */
-  obj = calloc(sizeof(struct ulp_dynobj), 1);
+  obj = calloc(1, sizeof(struct ulp_dynobj));
 
   if (read_memory((char *)&obj->link_map, sizeof(struct link_map), pid,
                   (Elf64_Addr)link_map_addr)) {
@@ -1249,29 +1250,22 @@ initialize_data_structures(struct ulp_process *process)
   ulp_error_t ret = 0;
 
   if (!process)
-    return 1;
+    return EINVAL;
 
   DEBUG("getting in-memory information about process %d.", process->pid);
 
   if (attach(process->pid)) {
-    DEBUG("Unable to attach to %d to read data.\n", process->pid);
-    ret = 1;
+    ret = ETARGETHOOK;
     goto detach_process;
   }
 
   ret = parse_main_dynobj(process);
   if (ret) {
-    WARN("unable to get in-memory information about the main executable: %s\n",
-         libpulp_strerror(ret));
-    ret = 1;
     goto detach_process;
   }
 
   ret = parse_libs_dynobj(process);
   if (ret) {
-    WARN("unable to get in-memory information about shared libraries: %s\n",
-         libpulp_strerror(ret));
-    ret = 1;
     goto detach_process;
   }
 
@@ -1280,7 +1274,7 @@ initialize_data_structures(struct ulp_process *process)
   if (read_memory((char *)&ulp_state, sizeof(ulp_state), process->pid,
                   process->dynobj_libpulp->state) ||
       ulp_state.load_state == 0) {
-    WARN("libpulp not ready (constructors not yet run). Try again later.");
+    DEBUG("libpulp not ready (constructors not yet run). Try again later.");
     ret = EAGAIN;
     goto detach_process;
   }
@@ -1451,7 +1445,7 @@ hijack_threads(struct ulp_process *process)
          *   Save all registers;
          *   Update the list.
          */
-        t = calloc(sizeof(struct ulp_thread), 1);
+        t = calloc(1, sizeof(struct ulp_thread));
         if (!t) {
           WARN("unable to allocate memory.");
           goto child_alloc_error;
@@ -2432,7 +2426,7 @@ error_path_context:
  * @return A `ulp_applied_patch` linked list.
  */
 static struct ulp_applied_patch *
-read_ulp_applied_patch(Elf64_Addr addr, pid_t pid)
+read_ulp_applied_patch(Elf64_Addr addr, pid_t pid, ulp_version_t version)
 {
   struct ulp_applied_patch *a_state;
 
@@ -2445,7 +2439,14 @@ read_ulp_applied_patch(Elf64_Addr addr, pid_t pid)
     return NULL;
   }
 
-  if (read_memory((char *)a_state, sizeof(*a_state), pid, addr)) {
+  size_t a_patch_size = sizeof(*a_state);
+
+  /* In pre 0.3.1 versions of libpulp, there was no timestamp.  */
+  if (version < ULP_VERSION_TRIPLET(0ULL, 3ULL, 1ULL)) {
+    a_patch_size -= sizeof(a_state->timestamp);
+  }
+
+  if (read_memory((char *)a_state, a_patch_size, pid, addr)) {
     WARN("error reading patch state.");
     free(a_state);
     return NULL;
@@ -2461,13 +2462,14 @@ read_ulp_applied_patch(Elf64_Addr addr, pid_t pid)
                 (Elf64_Addr)a_state->container_name);
   }
 
+
   /* ulp_applied_unit and ulp_applied_patch is not used, so don't read it. But
      set it to NULL to avoid dangling pointers.  */
 
   a_state->units = NULL;
   a_state->deps = NULL;
 
-  a_state->next = read_ulp_applied_patch((Elf64_Addr)a_state->next, pid);
+  a_state->next = read_ulp_applied_patch((Elf64_Addr)a_state->next, pid, version);
 
   return a_state;
 }
@@ -2527,7 +2529,8 @@ ulp_read_state(struct ulp_process *process)
     goto detach_process;
   }
 
-  ret = read_ulp_applied_patch((Elf64_Addr)state.patches, pid);
+  ret = read_ulp_applied_patch((Elf64_Addr)state.patches, pid,
+                               process->dynobj_libpulp->libpulp_version);
 
 detach_process:
   if (detach(process->pid)) {
