@@ -45,7 +45,7 @@
 bool enable_threading = false;
 static bool original_enable_threading;
 
-void insert_target_process(int pid, struct ulp_process **list);
+static int insert_target_process(int pid, struct ulp_process **list);
 
 static void *
 generate_ulp_list_thread(void *args)
@@ -131,6 +131,10 @@ process_list_next(struct ulp_process_iterator *it)
     symbols.  */
 static pthread_t process_list_thread;
 
+/** In case of `ulp patches -p <pid>`, we want to return an error for the
+    caller if something went wrong.  */
+static error_t any_error = ENONE;
+
 struct ulp_process *
 process_list_begin(struct ulp_process_iterator *it,
                    const char *procname_wildcard, const char *user_wildcard)
@@ -147,7 +151,7 @@ process_list_begin(struct ulp_process_iterator *it,
   if (isnumber(procname_wildcard)) {
     /* If wildcard is actually a number, then treat it as a PID.  */
     pid = atoi(procname_wildcard);
-    insert_target_process(pid, &it->last);
+    any_error = insert_target_process(pid, &it->last);
     it->now = it->last;
 
     /* Disable threading in this case.  */
@@ -180,10 +184,11 @@ process_list_end(struct ulp_process_iterator *it)
 {
   if (it->now == NULL) {
     release_ulp_process(it->last);
-    producer_consumer_delete(it->pcqueue);
     if (enable_threading) {
+      /* Make sure the threads stopped before destroying the queue.  */
       pthread_join(process_list_thread, NULL);
     }
+    producer_consumer_delete(it->pcqueue);
 
     /* In case threads were disabled because of some special case, then enable
        it now.  */
@@ -378,7 +383,11 @@ print_lib_patches(struct ulp_applied_patch *patch, const char *libname)
 
   while (patch) {
     if (!strcmp(libname, patch->lib_name)) {
-      printf("      livepatch: %s\n", get_basename(patch->container_name));
+      struct tm *timeinfo = localtime(&patch->timestamp);
+      printf("      livepatch: %s", get_basename(patch->container_name));
+      if (patch->timestamp != 0) {
+        printf(" loaded on %s", asctime(timeinfo));
+      }
       print_relevant_labels(patch->container_name);
     }
     patch = patch->next;
@@ -547,12 +556,19 @@ print_remote_err_status(struct ulp_process *p)
   change_color(TERM_COLOR_RESET);
 }
 
-void
-print_process(struct ulp_process *process, int print_buildid)
+static void
+print_process(struct ulp_process *process, int print_buildid,
+              bool only_livepatched)
 {
   struct ulp_dynobj *object_item;
   pid_t pid = process->pid;
   struct ulp_applied_patch *patch = ulp_read_state(process);
+
+  /* In case the flag `only_patched` is NULL, then skip process.  */
+  if (only_livepatched && patch == NULL) {
+    return;
+  }
+
   printf("PID: %d, name: %s\n", pid, get_process_name(process));
   print_remote_err_status(process);
   printf("  Livepatchable libraries:\n");
@@ -564,6 +580,11 @@ print_process(struct ulp_process *process, int print_buildid)
       printf("    in %s", object_item->filename);
       if (print_buildid)
         printf(" (%s)", buildid_to_string(object_item->build_id));
+      if (object_item->libpulp_version) {
+        const char *version =
+            ulp_version_as_string(object_item->libpulp_version);
+        printf(" (version %s)", version);
+      }
       printf(":\n");
 
       print_lib_patches(patch, object_item->filename);
@@ -604,7 +625,7 @@ has_libpulp_loaded(pid_t pid)
 /* Inserts a new process structure into LIST if the process identified
  * by PID is live-patchable.
  */
-void
+static int
 insert_target_process(int pid, struct ulp_process **list)
 {
   struct ulp_process *new = NULL;
@@ -615,14 +636,16 @@ insert_target_process(int pid, struct ulp_process **list)
   new->pid = pid;
   ret = initialize_data_structures(new);
   if (ret) {
-    WARN("error gathering target process information.");
+    WARN("error gathering target process information: %s",
+         libpulp_strerror(ret));
     release_ulp_process(new);
-    return;
+    return ret;
   }
   else {
     new->next = *list;
     *list = new;
   }
+  return 0;
 }
 
 /** @brief Prints all the info collected about the processes in `process_list`.
@@ -637,7 +660,7 @@ print_process_list(struct ulp_process *process_list, int print_buildid)
 
   process_item = process_list;
   while (process_item) {
-    print_process(process_item, print_buildid);
+    print_process(process_item, print_buildid, false);
     process_item = process_item->next;
   }
 }
@@ -646,6 +669,7 @@ int
 run_patches(struct arguments *arguments)
 {
   bool print_build_id = arguments->buildid;
+  bool only_livepatched = arguments->only_livepatched;
   ulp_quiet = arguments->quiet;
   ulp_verbose = arguments->verbose;
   enable_threading = !arguments->disable_threads;
@@ -653,11 +677,12 @@ run_patches(struct arguments *arguments)
   const char *user_wildcard = arguments->user_wildcard;
 
   struct ulp_process *p;
+  any_error = ENONE;
 
   FOR_EACH_ULP_PROCESS_FROM_USER_WILDCARD(p, process_wildcard, user_wildcard)
   {
-    print_process(p, print_build_id);
+    print_process(p, print_build_id, only_livepatched);
   }
 
-  return 0;
+  return any_error == 0 ? 0 : 1;
 }
